from constants import *
import graphics

class menuItem:
    """An item in a menu"""
    def __init__(self,text):
        self.text = text
    def select(self,menu):
        pass

class normalMenuItem(menuItem):
    """Selecting this calls a function"""
    def __init__(self,text,controller,function):
        menuItem.__init__(self,text)
        self.controller = controller
        self.function = function
    def select(self,menu):
        function = getattr(self.controller,self.function)
        if callable(function):
            function() #call its function
        

class subMenuItem(menuItem):
    """Selecting this changes the current menu"""
    def __init__(self,text,menu):
        menuItem.__init__(self,text)
        self.menu = menu
    def select(self,menu):
        if self.menu == menu.parentMenu:
            #go back to the parent menu
            menu.parentMenu.activeMenu = menu.parentMenu
        else:
            #go to a submenu
            menu.activeMenu = self.menu
    

class menu:
    """Simple menu which is controlled by the keyboard"""
    def __init__(self,parent=None):
        self.items = []
        self.activeMenu = self #if set to one of the child menus then this will be shown instead
        self.parentMenu = parent
        self.selected = 0 #selected item (shown in middle)
        self.backButton = None
        if self.parentMenu:
            self.backButton = subMenuItem("Back", self.parentMenu)

    def reset(self):
        """Go back to the top level menu"""
        self.selected = 0
        if self.activeMenu != self:
            self.activeMenu.reset()
            self.activeMenu = self

    def addItem(self,item):
        """Add an item to the menu"""
        self.items.append(item)

    def draw(self,graphicsThing, fontThing):
        if self.activeMenu == self:
            width = graphicsThing.screenWidth()
            height = graphicsThing.screenHeight()

            #draw items above the current selected item
            above = len(self.items[:self.selected])
            for i in range(above):
                text = self.items[i].text
                fontThing.writeText(text,"VeraBd.ttf",24,(0,0,0),(width/2,height/2-(above-i)*30),centered=True,save=True)

            #draw selected item in the middle
            if self.selected < len(self.items):
                text = self.items[self.selected].text
            else:
                text = self.backButton.text
            fontThing.writeText(text,"VeraBd.ttf",24,(0,0,200),(width/2,height/2),centered=True,save=True)

            remaining = []
            if len(self.items) - 1 > self.selected:
                remaining.extend(self.items[self.selected+1:]) #add all the items after the selected one
            if self.backButton and self.selected < len(self.items):
                remaining.extend([self.backButton]) #add the back button at the bottom

            for i in range(len(remaining)):
                text = remaining[i].text
                fontThing.writeText(text,"VeraBd.ttf",24,(0,0,0),(width/2,height/2+(i+1)*30),centered=True,save=True)
        else:
            self.activeMenu.draw(graphicsThing,fontThing)

    def input(self,inputThing):
        """Receive input"""
        if self.activeMenu == self:
            if inputThing.mousePressed(): #cant be bothered with this, use keyboard for menus
                pass

            last = len(self.items) - 1
            if self.backButton:
                last += 1

            if inputThing.keyPressed("up"): #up has been pressed since last time function was called
                self.selected -= 1
                if self.selected < 0:
                    self.selected = last

            elif inputThing.keyPressed("down"): #down has been pressed since last time function was called
                self.selected += 1
                if self.selected > last:
                    self.selected = 0

            elif inputThing.keyPressed("enter"): #enter has been pressed
                if self.selected == len(self.items) and self.backButton:
                    self.backButton.select(self)
                else:
                    item = self.items[self.selected]
                    item.select(self)

        else:
            self.activeMenu.input(inputThing)
