import graphics
import color
import math
from constants import *

class rope:
    """Represents a string"""
    def __init__(self, pos, bottomEdge, speed, length):
        hsv = color.randHSV()
        self.cut = False
        self.gap = [0, 0] #when it gets cut the two sides move apart, this describes where the gap starts and finishes
        self.color = color.hsv2rgb(hsv)
        self.pos = [pos[0],pos[1]] #position of leftmost point
        self.bottomEdge = bottomEdge #if the string goes past here it is removed
        self.speed = speed #pixels per ms
        self.length = length #how long it starts off (should be the same for all strings)
        self.hue = hsv[0]
        self.requiredLength = int(hsv[0]/360 * length)        

    def draw(self,graphics):
        """Draw the string"""
        if self.cut: #draw two pieces
            start1 = (int(self.pos[0]), int(self.pos[1]))
            finish1 = (self.gap[0], start1[1])
            start2 = (self.gap[1], start1[1])
            finish2 = (start1[0]+self.length,start1[1])
            graphics.drawLine(self.color, start1, finish1, 2)
            graphics.drawLine(self.color, start2, finish2, 2)
        else:
            start = (int(self.pos[0]), int(self.pos[1]))
            finish = (start[0]+self.length,start[1])
            graphics.drawLine(self.color, start, finish, 2)
            graphics.drawLine((100,100,100), (start[0],start[1]+2), (start[0]+conveyorWidth-1,start[1]+2), 1)
            graphics.drawLine((140,140,140), (start[0]+conveyorWidth,start[1]+10), (finish[0]-conveyorWidth,start[1]+10), 4)
            graphics.drawLine((100,100,100), (finish[0]-conveyorWidth+1,start[1]+2), (finish[0],start[1]+2), 1)

    def move(self,dt):
        """move(dt)
        Move the string. Displacement = speed * dt"""
        self.pos[1] += self.speed*dt
        if self.cut:
            self.gap[0] -= removalSpeed*dt
            self.gap[1] += removalSpeed*dt

    def offscreen(self):
        """offscreen()
        checks to see if the rope is offscreen"""
        if self.pos[1] > self.bottomEdge:
            return True;
        else:
            return False

    def isCut(self):
        """Has the string been cut yet?"""
        return self.cut

    def isGone(self):
        """Have the pieces been removed yet?"""
        if self.gap[0] < self.pos[0] and self.gap[1] > self.length:
            return True
        else:
            return False

    def cutAt(self,x):
        """"Cut the string at x and return the perecentage error"""
        self.cut = True
        self.gap = [x,x+1]

        error = math.fabs(self.requiredLength-x)

        #if its a reddish colour then check treat the left edge as being joined to the right edge
        if self.requiredLength < self.length/6 and x > 5*self.length/6:
            error2 = self.requiredLength+self.length - x
            if error2 < error:
                error = error2
        elif self.requiredLength > 5*self.length/6 and x < self.length/6:
            error2 = x+self.length - self.requiredLength
            if error2 < error:
                error = error2

        return float(error)/self.length * 100


    def collide(self,x):
        """Assuming its been cut, is the string still in the way of the laser at x"""
        if self.gap[0] <= x <= self.gap[1]:
            return False
        else:
            return True
        
    def getPos(self):
        """Get vertical position"""
        return int(self.pos[1])
